package Isconf::Crontab;
####################################################################################
####################################################################################
# Isconf::Crontab
####################################################################################
####################################################################################

use Exporter;
use Isconf;
use vars qw(%locations @ISA $VERSION);

@ISA = qw(Isconf);
use strict;

# this lets cvs manage our version number
$VERSION = sprintf("%d.%02d", q$Revision: 1.3 $ =~ /(\d+)\.(\d+)/);

# this has to be passed the user name;
%locations = (
	'default'	=> sub { if (@_) { return "/var/spool/cron/crontabs/$_[0]"; } },
);
=head1 NAME

   Isconf::Crontab - module for working with Unix crontabs

=head1 SYNOPSIS

   my $crontab = Isconf::Crontab->new('root');

	$crontab->add($job);
	$crontab->delete($otherjob);
	$crontab->update();

=head1 DESCRIPTION

B<Isconf::Crontab> abstracts working with Unix crontabs.  At this point
it requires that B<crontab -l> list the current crontab, and also requires
the ability to pipe STDIN to B<crontab> for writing.

B<Isconf::Crontab> is entirely object-oriented.  All methods require
that they be called from an object.

=head1 METHODS

=over 4

=item add

	Add a string of text as a cron job.

=cut

####################################################################################
# Isconf::Crontab::add
sub add
{
	my $obj = shift;

	my ($text,@crontab);
	$text = shift;

	@crontab = $obj->crontab();
	unless (grep /^$text$/, @crontab)
	{
		push @crontab, $text;
	}
	$obj->crontab(@crontab);
}
# Isconf::Crontab::add
####################################################################################

=item crontab

	If necessary, load the crontab, and return its current (cached) contents.

=cut

####################################################################################
# Isconf::Crontab::crontab
sub crontab
{
	my $obj = shift;
	if (@_)
	{
		@{ $obj->{'crontab'} } = @_;
	}
	unless (exists $obj->{'crontab'})
	{
		open CRON, "crontab -l " . $obj->user() . "|" or die "Could not open crontab command for reading";
		@{ $obj->{'crontab'} } = <CRON>;
		close CRON;
	}

	if (wantarray)
	{
		return @{ $obj->{'crontab'} };
	}
}
# Isconf::Crontab::crontab
####################################################################################

=item delete

	Delete an existing cron job.  Must exactly match the job.
	Actually, it doesn't currently have to match the job exactly (that is,
	it's not set with line anchors).  I don't know if that should be fixed...

=cut

####################################################################################
# Isconf::Crontab::delete
sub delete
{
	my $obj = shift;
	my ($text,@oldtab,@newtab);
	$text = shift;

	@oldtab = $obj->crontab();
	foreach (@oldtab)
	{
		if (! /\Q$text\E/) { push @newtab, $_; }
	}
	#@newtab = grep !/$text/, @oldtab;

	unless (@newtab eq @oldtab)
	{
		$obj->debug(10,"committing delete");
		$obj->crontab(@newtab);
	}
}
# Isconf::Crontab::delete
####################################################################################

=item new

	Create the crontab object; does not actually load the crontab yet.

=cut

####################################################################################
# Isconf::Crontab::new
sub new
{
	my ($class,$obj,$user,$uname,);
	$class = shift;
	$obj = bless {}, $class;

	$user = shift or return;

	$obj->user($user);
	#chuser($obj->uid($user));
	$obj->debug(10, "User:",$obj->user(),"UID:",$obj->uid());
	$< = $obj->uid();

	return $obj;
}
# Isconf::Crontab::new
####################################################################################

=item replace

	Replace $_[0] with $_[1] within the crontab.

=cut

####################################################################################
# Isconf::Crontab::replace
sub replace
{
	my $obj = shift;
	my ($ntext,$otext,@crontab,$changes);
	($ntext,$otext) = @_;

	@crontab = $obj->crontab();
	#@newtab = grep !/$text/, @oldtab;
	$changes = grep { /^\Q$otext\E$/ && s/\Q$_\E/$ntext/; } @crontab;

	if ($changes)
	{
		$obj->crontab(@crontab);
	}
}
# Isconf::Crontab::replace
####################################################################################

=item uid

	Return the uid associated with the user.  Used to change the UID,
	to allow use with users other than the given user and root.

=cut

####################################################################################
# Isconf::Crontab::uid
sub uid
{
	my $obj = shift;
	my ($uid);
	unless ($obj->{'uid'})
	{
		(undef,undef,$uid,undef) = getpwnam($obj->user()) or return;;
		$obj->{'uid'} = $uid;
	}
	$obj->debug(10,"uid is $obj->{'uid'}\n");
	return $obj->{'uid'};
}
# Isconf::Crontab::uid
####################################################################################

=item update

	Write the crontab out.

=cut

####################################################################################
# Isconf::Crontab::update
sub update
{
	my $obj = shift;
	my $line;
	open CRON, "|crontab" or die "Could not open crontab command for writing\n";
	foreach $line ($obj->crontab())
	{
		if ($line !~ /\n$/) { $line .= "\n"; }

		print CRON $line;
	}
	close CRON;
}
# Isconf::Crontab::update
####################################################################################

=item user

	If necessary, set the username, and return its current setting.

=cut

####################################################################################
# Isconf::Crontab::user
sub user
{
	my $obj = shift;
	if (@_)
	{
		$obj->{'user'} = shift;
	}
	return $obj->{'user'} || '';
}
# Isconf::Crontab::user
####################################################################################

=back

=head1 SEE ALSO

L<Cat>,
L<Isconf>

=head1 AUTHOR

Luke A. Kanies, luke.kanies@cat.com

=for html <hr>

I<$Id: Crontab.pm,v 1.3 2003/07/24 23:56:07 loosifer Exp $>

=cut

1;
