package Isconf::File;
####################################################################################
####################################################################################
# Isconf::File
####################################################################################
####################################################################################

use vars qw(@ISA $VERSION);

@ISA = qw(Isconf);

use Isconf::File::Blank;
use Isconf::File::Comment;
use Isconf::File::Item;
use Isconf::File::Target;
use Isconf::File::Targethead;
use Isconf::File::Variable;

$Isconf::File::GRAMMAR = <<'GRAMMAR';
file: <skip:"[ \t]*"> sectorcomm(s) /\s*\z/
{
	$item{'sectorcomm'};
}
sectorcomm: section { $item{'section'}; }
	| comment | var /\n/ { $item{'var'}; } | blank
section: targethead /\n\s+/ itemsorvars(s?)
{
	Isconf::File::Target->new(
		targethead => $item{'targethead'},
		itemsorvars => $item{'itemsorvars'},
	);
}
targethead: /^/ target(s) /:/ prereq(s?) comment(?) {
	my (%args,$comment);
	if (exists $item{'target'} and $item{'target'})
	{
		$args{'names'} = $item{'target'};
	}

	foreach (qw(prereq comment))
	{
		if (exists $item{$_} and $item{$_})
		{
			$args{$_} = $item{$_};
		}
	}

	if (exists $args{'comment'} and $args{'comment'})
	{
		$comment = shift @{ $args{'comment'} };
		if ($comment)
		{
			$args{'comment'} = $comment;
		}
		else
		{
			delete $args{'comment'}
		}
	}

	Isconf::File::Targethead->new(%args);
}
target: /([-_\.\w\/]+)/ {$1;}
prereq: term <skip:""> /\([^\n]*\)/ | term
term: /[-_\${}\w\/\.]+/
itemsorvars: item | var /\n/ { $item{'var'}; }
item: iname var(s?) comment(?) /\n/
{
	my (%args,$comment);

	if (exists $item{'iname'} and $item{'iname'})
	{
		$args{'name'} = $item{'iname'};
	}
	if (exists $item{'var'} and $item{'var'})
	{
		$args{'vars'} = $item{'var'};
	}
	if (exists $item{'comment'} and $item{'comment'})
	{
		$comment = shift @{ $item{'comment'} };

		if ($comment)
		{
			$args{'comment'} = $comment
		}
		else
		{
			delete $args{'comment'}
		}
	}

	Isconf::File::Item->new(%args);
}
iname: prereq
var: ivarname <skip:""> /=/ ivarval
{
	Isconf::File::Variable->new(
		$item{'ivarname'} => $item{'ivarval'},
	);
}
ivarname: /[-_\.\w]+/
ivarval: /'[^']+'/ | /"[^"]+"/ | /\S+/
comment: /([^\n]*#[^\n]*)/
{
	Isconf::File::Comment->new($1);
}
blank: /(\n+)/
{
	Isconf::File::Blank->new($1);
}
GRAMMAR

####################################################################################
# Isconf::File::new
sub new
{
	my ($class,$obj,$objlist,$ref,$file,@lines,$comment,$refclass);
	$class = shift;
	$obj = bless {}, $class;

	$file = shift;

	$obj->name($file);

	# try for the new parser....
	eval {
		require Isconf::Parse;
	};

	if ($@)
	{
		# if we don't get it, then use the old one
		warn "Could not load Isconf::Parse: $@\nusing Parse::RecDescent\n";
		eval {
			require Parse::RecDescent;
		};
		if ($@) {
			warn "Could not load Parse::RecDescent either: $@\n";
			warn "No valid parser found.  Exiting\n";
			exit;
		}
		delete $INC{'Isconf/Parse.pm'};


		open TYPES, $file or die "Could not open $file: $!\n";
		@lines = <TYPES>;
		close TYPES;

		$obj->{'parser'} = Parse::RecDescent->new($Isconf::File::GRAMMAR) or
			die "Bad grammar\n";
		$objlist = $obj->{'parser'}->file("@lines");
		defined $objlist or die "Bad text; $file failed to parse\n";

		undef @lines;
	
		$::DEBUG && $obj->debug(10,"new $file");
	
		$::DEBUG && $obj->debug(10,"new: objlist $objlist");
		$::DEBUG && $obj->debug(10,"new: objlist $obj->{'list'}");
	
		foreach $ref (@$objlist)
		{
			$refclass = ref $ref;
			$::DEBUG && $obj->debug(8,"new: working on ref $ref");
			$refclass =~ /Isconf::File::Variable/ and do
			{
				$obj->{'vars'}->{'hash'}->{$ref->{'name'}} = $ref;
				push @{ $obj->{'vars'}->{'ary'} }, $ref;
				next;
			};
	
			$refclass =~ /Isconf::File::Target/ and do
			{
				# okay....
				# because our format supports multiple target names before
				# the colon (just like make) but our objects really only
				# want one name, we have to loop over all of the names
				# associated with the target and add a ref under
				# each name
				#   yes, this should be more abstracted, but it's just not right now...
				foreach (@{ $ref->{'targethead'}->{'names'} })
				{
					$obj->{'targets'}->{'hash'}->{$_} = $ref;
				}
				push @{ $obj->{'targets'}->{'ary'} }, $ref;
				next;
			};
	
			$refclass =~ /Isconf::File::Blank/ and do
			{
				next;
			};
	
			$refclass =~ /Isconf::File::Comment/ and do
			{
				$::DEBUG && $obj->debug(8,"new: comment ref $ref [$ref->{'comment'}]");
				next;
			};
	
			$refclass =~ /ARRAY/ and do
			{
				$comment = shift @$ref;
				$::DEBUG && $obj->debug(8,"new: comment $comment ref $ref");
				$ref = $comment;
				next;
			};
			$::DEBUG && $obj->debug(8,"new: missed ref $refclass");
		}
	}
	else
	{
		# otherwise use the new one
		# yay, speed!
		#warn "using Isconf::Parse\n";
		$objlist = Isconf::Parse::parse($file);
		defined $objlist or die "Bad text; $file failed to parse\n";

		my (@variables,@stanzas,);
		# take all of the objects and add them to our file
		foreach $ref (@$objlist)
		{
			$refclass = ref $ref;
			$::DEBUG && $obj->debug(8,"new: working on ref $ref");
			$refclass =~ /Isconf::File::Variable/ and do
			{
				push @variables, $ref;
				next;
			};
	
			$refclass =~ /Isconf::File::Stanza/ and do
			{
				# okay....
				# because our format supports multiple target names before
				# the colon (just like make) but our objects really only
				# want one name, we have to loop over all of the names
				# associated with the target and add a ref under
				# each name
				#   yes, this should be more abstracted, but it's just not right now...
				#foreach ($ref->names())
				#{
				#	$obj->{'targets'}->{'hash'}->{$_} = $ref;
				#}
				push @stanzas, $ref;
				next;
			};
	
			$refclass =~ /Isconf::File::Blank/ and do
			{
				next;
			};
	
			$refclass =~ /Isconf::File::Comment/ and do
			{
				$::DEBUG && $obj->debug(8,"new: comment ref $ref [$ref->{'comment'}]");
				next;
			};
	
			$refclass =~ /ARRAY/ and do
			{
				$comment = shift @$ref;
				$::DEBUG && $obj->debug(8,"new: comment $comment ref $ref");
				$ref = $comment;
				next;
			};
			$::DEBUG && $obj->debug(8,"new: missed ref $refclass");
		}
		$obj->setvars(@variables);
		$obj->settargets(@stanzas);
	}
	#print "adding $objlist as list of objects (", join(" ", @$objlist), ")\n";
	$obj->{'list'} = $objlist;

	return $obj;
}
# Isconf::File::new
####################################################################################

####################################################################################
# Isconf::File::name
sub name
{
	my $obj = shift;
	$::DEBUG && $obj->debug(6,"name: arguments: @_");
	if (@_)
	{
		$::DEBUG && $obj->debug(6,"name: setting @_");
		$obj->{'name'} = shift;
	}
	if (exists $obj->{'name'})
	{
		$::DEBUG && $obj->debug(6,"name: returning $obj->{'name'}");
		return $obj->{'name'};
	}
	else
	{
		$::DEBUG && $obj->debug(6,"could not find name");
		return;
	}
}
# Isconf::File::name
####################################################################################

####################################################################################
# Isconf::File::prereqs

=item prereqs

	This routine isn't used in Isconf::File; it's only used by the Target
	and Targethead modules, but I put it here for inheritance reasons.

=cut

sub prereqs
{
	my $obj = shift;

	if (@_)
	{
		if (ref $_[0] eq 'ARRAY')
		{
			$obj->{'prereq'} = shift;
		}
		else
		{
			$obj->{'prereq'} = [ @_ ];
		}
	}

	if (exists $obj->{'prereq'})
	{
		if (wantarray)
		{
			return @{ $obj->{'prereq'} };
		}
		else
		{
			return $obj->{'prereq'};
		}
	}
	else
	{
		return;
	}
}
# Isconf::File::Target::prereqs
####################################################################################

####################################################################################
# Isconf::File::print
sub print
{
	my $obj = shift;
	my ($class,$oclass);
	$::DEBUG && $obj->debug(10,'print');
	my ($string,$var,%args);

	foreach $var (@{ $obj->{'list'} })
	{
		if ($var =~ /Blank/) { next; }
		if ($class){ $oclass = $class; }
		$class = ref $var;
		if ($oclass and $class ne $oclass)
		{
			$string .= "\n";
		}
		$::DEBUG && $obj->debug(8,"print: adding $var");
		$string .= $var->print(@_) . "\n";
	}
	

	return $string;
}
# Isconf::File::print
####################################################################################

####################################################################################
# Isconf::File::settargets
sub settargets
{
	my $obj = shift;
	$::DEBUG && $obj->debug(6,"settargets");
	if (@_)
	{
		foreach my $ref (@_)
		{
			foreach my $name ($ref->names())
			{
				$obj->{'targets'}->{'hash'}->{$name} = $ref;
			}
		}
	}
	return 1;
}
# Isconf::File::settargets
####################################################################################

####################################################################################
# Isconf::File::setvars
sub setvars
{
	my $obj = shift;
	$::DEBUG && $obj->debug(6,"setvars");
	if (@_)
	{
		foreach my $ref (@_)
		{
			$obj->{'vars'}->{'hash'}->{$ref->name()} = $ref;
		}
	}
	return 1;
}
# Isconf::File::settargets
####################################################################################

####################################################################################
# Isconf::File::timestamp
sub timestamp
{
	if ($::STAMP) { $::STAMP++; }
	else
	{
		my $format = '%Y%m%d%H%M%S';
		my $date = qx|date +$format|;
		chomp $date;
		$::STAMP = $date . "00";
	}
	return $::STAMP;
}
# Isconf::File::timestamp
####################################################################################

####################################################################################
# Isconf::File::targets

=item targets

	Accepts a single argument, the name of a target.  If passed an
	argument, returns a reference to a target, if it exists, or
	returns nothing.  If passed nothing, returns a list of existing
	targets, if any.  All returns are Isconf::File::Target objects.

	Just calls Isconf::lists, because it's such a heavily used type of
	routine.

=cut

sub targets
{
	my $obj = shift;
	$::DEBUG && $obj->debug(6,'targets');
	return $obj->lists('targets',@_);
}
# Isconf::File::targets
####################################################################################



=head1 NAME

   Isconf::File - Modules for parsing Isconf 3l config files

=head1 SYNOPSIS

   Isconf::File

=head1 DESCRIPTION

B<Isconf::File>

=head1 OPTIONS

=over 4


=item dummy

	E

=back

=head1 SEE ALSO

L<Cat>,
L<Isconf>

=head1 AUTHOR

Luke A. Kanies, luke.kanies@cat.com

=for html <hr>

I<$Id: File.pm,v 1.6 2003/10/23 16:26:11 loosifer Exp $>

=cut
1;
